/*
 * FlexWire.c
 *
 *  Created on: Apr 15, 2019
 *      Author: a0222913
 */

#include <msp430.h>
#include <FlexLED.h>

extern unsigned int responseData[]; //store the response data bytes
extern unsigned int receiveByteNum; //store the received byte number
extern unsigned int CANused; //flag of whether CAN transceiver is used or not

extern char dmaTxData[100];
//extern char dmaRxData[100];

void FlexWrite(char DEV_ADDR_x, char registerAddr, char DATA_BYTES[], char DATA_LENGTH_x, unsigned int checkResponse)
{
    char DEV_ADDR=0x00, REG_ADDR;
    char commandFrame[12];// one longest command frame length is 12
    char i, dataLength, frameLength, responseLength;

    DEV_ADDR = (FLEX_Write | DATA_LENGTH_x | DEV_ADDR_x);
    REG_ADDR = registerAddr;

    commandFrame[0] = 0x55; //SYNC byte
    commandFrame[1] = DEV_ADDR;
    commandFrame[2] = REG_ADDR;

    switch (DATA_LENGTH_x)
    {
    case 0:
        {
            dataLength = 1;
            break;
        }
    case 16:
        {
            dataLength = 2;
            break;
        }
    case 32:
        {
            dataLength = 4;
            break;
        }
    case 48:
        {
            dataLength = 8;
            break;
        }
    default : break;
    }

    for(i=0;i<dataLength;i++)
    {
        commandFrame[i+3] = DATA_BYTES[i];
    }

    commandFrame[i+3] = CRC_LUT(commandFrame,i+3); //calculate CRC of all the command frame bytes

    frameLength = i+4; //store the entire command frame byte length

    responseLength = frameLength+2; //as CAN Transceiver is connected, all the sent data will also loop back to receive terminal of MCU
                                    //so, the number of response number should be (Num of sent data + Num of real response data + CRC)

    for(i=0;i<frameLength;i++)
    {
        dmaTxData[i]=commandFrame[i];
    }
    dmaTransfer(frameLength, responseLength, checkResponse); //data will be stored in dmaRxData[]

    // if checkRepsonse is not set, a minimum wait time is needed before writing next command frame, to make sure next writing command is after response finished sending
    // the minimum wait time can be calculated by the formula below
    // minimum wait time = response length * 10 *1/baurdrate

}

void FlexRead(char DEV_ADDR_x, char registerAddr, char DATA_LENGTH_x)
{
    char DEV_ADDR=0x00, REG_ADDR;
    char commandFrame[4];
    unsigned int dataLength, responseLength, i;
    DEV_ADDR = (FLEX_Read | DATA_LENGTH_x | DEV_ADDR_x);
    REG_ADDR = registerAddr;

    commandFrame[0] = 0x55; //SYNC byte
    commandFrame[1] = DEV_ADDR;
    commandFrame[2] = REG_ADDR;

    switch (DATA_LENGTH_x)
    {
    case 0:
        {
            dataLength = 1;
            break;
        }
    case 16:
        {
            dataLength = 2;
            break;
        }
    case 32:
        {
            dataLength = 4;
            break;
        }
    case 48:
        {
            dataLength = 8;
            break;
        }
    default : break;
    }

    commandFrame[3] = CRC_LUT(commandFrame,3); //calculate CRC of all the command frame bytes

    responseLength = 4+dataLength+1;//as CAN Transceiver is connected, all the sent data will also loop back to receive terminal of MCU
    //so, the number of response number should be (Num of sent data + Num of real response data + CRC)


    for(i=0;i<4;i++)
    {
        dmaTxData[i]=commandFrame[i];
    }
    dmaTransfer(4, responseLength, TRUE); //data will be stored in dmaRxData[]

}
