/*
 * Timer.c
 *
 *  Created on: Apr 9, 2019
 *      Author: a0222913
 */
#include <msp430.h>
#include <FlexLED.h>

unsigned int delayFlag=0;
extern unsigned int timeOutFlag;

void oneMHzACLK() //configure the ACLK = SMCLK as 1MHz
{
    UCSCTL4 |= SELA_4;                       // ACLK=DCO, SMCLK=MCLK=DCODIV
}

void fourMHzACLK() //configure the ACLK as 4MHz
{
     P5SEL |= BIT2 + BIT3;                      // P5.2 -> XT2IN, P5.3 -> XT2OUT
     UCSCTL6 &= ~XT2OFF;                       // Enable XT2
     UCSCTL3 |= SELREF_2;                      // FLLref = REFO
                                                // Since LFXT1 is not used,
                                                // sourcing FLL with LFXT1 can cause
                                                // XT1OFFG flag to set
      UCSCTL4 |= SELA_2;                        // ACLK=REFO,SMCLK=DCO,MCLK=DCO

      // Loop until XT1,XT2 & DCO stabilizes - in this case loop until XT2 settles
      do
      {
        UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG);
                                                // Clear XT2,XT1,DCO fault flags
        SFRIFG1 &= ~OFIFG;                      // Clear fault flags
      }while (SFRIFG1&OFIFG);                   // Test oscillator fault flag

      UCSCTL6 &= ~XT2DRIVE0;                    // Decrease XT2 Drive according to
                                                // expected frequency
      UCSCTL4 |= SELA_5+SELS_4;                       // ACLK=XT2, SMCLK=MCLK=DCODIV

}

void ucsInitial()
{
   fourMHzACLK(); //set ACLK as 4MHz
   //oneMHzACLK(); //set ACLK as 1MHz
}


void timerInitial()
{
    TA0CTL |= TASSEL_2 + ID_3 + MC_0; //select SMCLK clock source, select /8 divider, select stop mode and enable Timer_A interrupt
    TA1CTL |= TASSEL_2 + ID_3 + MC_0; //select SMCLK clock source, select /8 divider, select stop mode and enable Timer_A interrupt
}

void delay(int delayTime) //132.5 == 1ms
{
    delayFlag=0;
    TA0CTL |= TACLR; //clear timer
    TA0CCR0 = delayTime;
    TA0CTL |= MC_1 + TAIE; //select up mode to start counting
    do
    {
        ;
    }
    while(delayFlag==0);//wait until timer flow

}

void watchDog(int timeOut)//132.5 == 1ms
{
    timeOutFlag=0;
    TA1CTL |= TACLR; //clear timer
    TA1CCR0 = timeOut;
    TA1CTL |= MC_1 + TAIE; //select up mode to start counting
}

#pragma vector = TIMER0_A1_VECTOR
__interrupt void Timer0_A(void)
{
    TA0CTL &= ~TAIE; //disable interrupt
    TA0CTL &= ~TAIFG; //clear flag
    TA0CTL &= ~MC_1; //stop Timer
    delayFlag=1; //set delay flag
}

#pragma vector = TIMER1_A1_VECTOR
__interrupt void Timer1_A(void)
{
    TA1CTL &= ~TAIE;
    TA1CTL &= ~TAIFG;
    TA1CTL &= ~MC_1; //stop Timer
    timeOutFlag=1; //set timeout flag
}

