/*
 * dma.c
 *
 *  Created on: Jun 24, 2019
 *      Author: a0222913
 */
#include <stdint.h>
#include <msp430.h>
#include <FlexLED.h>

char dmaTxData[100];
char dmaRxData[100];
extern unsigned int timeOutFlag;

void dmaInitial()
{
    DMACTL0 = DMA1TSEL_16+DMA0TSEL_17;        // DMA0 - UCA0TXIFG
                                               // DMA1 - UCA0RXIFG
     // Setup DMA0
     __data20_write_long((uintptr_t) &DMA0SA,(uintptr_t) dmaTxData);
                                               // Source block address
     __data20_write_long((uintptr_t) &DMA0DA,(uintptr_t) &UCA0TXBUF);
                                               // Destination single address
     DMA0CTL = DMASRCINCR_3+DMASBDB+DMALEVEL;  // increase source address, source byte to destination byte, level sensitive

     // Setup DMA1
     __data20_write_long((uintptr_t) &DMA1SA,(uintptr_t) &UCA0RXBUF);
                                               // Source single address
     __data20_write_long((uintptr_t) &DMA1DA,(uintptr_t) dmaRxData);
                                               // Destination block address
     DMA1CTL = DMADSTINCR_3+DMASBDB+DMALEVEL;  // increase destination address, source byte to destination byte, level sensitive
}

void dmaTransfer(unsigned int dmaTxSize, unsigned int dmaRxSize, unsigned int checkResponse)
{
    DMA0SZ = dmaTxSize;
    DMA1SZ = dmaRxSize;

    DMA0CTL |= DMAEN; //enable Tx DMA

    if(checkResponse==TRUE)
    {
        DMA1CTL |= DMAEN; //enable Rx DMA
        // in order to avoid program entering infinite waiting for receiving specified numbers of response,
        //we set a longest wait time, if the response wait time exceed the time, the program will also exit the wait
        watchDog(26500); //launch timer and set wait time = 200ms

        while((!(DMA1CTL&DMAIFG))&&(timeOutFlag == 0)); //received all response byte or the wait time exceeds the specified time

        if(timeOutFlag==1) //have not received all specified numbers of response in defined time
        {
            // You can take some action once the response has not been received
            //Below is an example: turn on one red LED on the launch board
            P1OUT |= BIT0; //set P1.0 high to turn on the red LED in the launch pad

        }
        else
        {
            P1OUT &= ~BIT0; //successfully received response, clear the red LED
        }
        DMA1CTL &= ~(DMAEN + DMAIFG); //disable DMA1
        DMA0CTL &= ~(DMAEN + DMAIFG);//disable DMA0
    }
    else
    {
        // in order to avoid program entering infinite waiting for sending specified numbers of command bytes,
        //we set a longest wait time, if the response wait time exceed the time, the program will also exit the wait
        watchDog(26500); //launch timer and set wait time = 200ms
        while((!(DMA0CTL&DMAIFG))&&(timeOutFlag == 0)); //send all command bytes or the wait time exceeds the specified time
        if(timeOutFlag==1) //have not sent all specified numbers of command bytes in defined time
        {
            // You can take some action once the command bytes have not been sent
            //Below is an example: turn on one red LED on the launch board
            P1OUT |= BIT0; //set P1.0 high to turn on the red LED in the launch pad

        }
        else
        {
            P1OUT &= ~BIT0; //successfully sent all command bytes, clear the red LED
        }
        DMA0CTL &= ~(DMAEN + DMAIFG);//disable DMA0
    }
}
